using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;

using Syncfusion.Windows.Forms.Grid;


namespace testing
{
	/// <summary>
	/// Summary description for GridPrintToFitDocument.
	/// </summary>
	public class GridPrintToFitDocument : GridPrintDocument
	{
		private GridControlBase _grid;
		private string _reportName;
		private string _reportStartDate;
		private string _reportEndDate;
		private int	   _rowIndex = 0;
		private int	   _headerHeight = 0;
		private int	   _pageCount = 0;
		private int	   _printCount = 0;

		public GridPrintToFitDocument(GridControlBase grid, bool printPreview)
			: base(grid, printPreview)
		{
			_grid = grid;

// NONE OF THESE CHANGES ANYTHING
//			_grid.Model.Properties.PrintRowHeader = false;
//			((GridDataBoundGrid)_grid).Properties.PrintRowHeader = false;
//			_grid.Model.QueryColWidth += new GridRowColSizeEventHandler(grid_QueryColWidth);
		}

		public int GetPrintCount()
		{
			return _printCount;
		}

		public void SetReportDetails(string reportName, string reportStartDate, string reportEndDate)
		{
			this._reportName = reportName;
			this._reportStartDate = reportStartDate;
			this._reportEndDate = reportEndDate;
		}

		private void grid_QueryColWidth(object sender, Syncfusion.Windows.Forms.Grid.GridRowColSizeEventArgs e)
		{
			
			if(_grid.PrintingMode)
			{
				if (e.Index == 0)
				{
					e.Size = 0;
					e.Handled = true;
				}
			}
		
		}

		protected override void OnPrintPage(System.Drawing.Printing.PrintPageEventArgs ev)
		{
			if (_pageCount == 0)
			{
				_printCount++;
				_grid.Selections.Clear();
			}
			
/* DOES NOTHING
			_grid.Model.Properties.PrintRowHeader = false;
			_grid.Model.Properties.PrintHorzLines = false;
			_grid.Model.Properties.PrintVertLines = false;
			_grid.Model.Properties.PrintFrame = false;
			_grid.Model.Properties.PrintColHeader = false;
*/			_grid.Model.Properties.PrintRowHeader = false;


			_pageCount++;
			if (_rowIndex == 0)
			{
				_headerHeight = _grid.GetRowHeight(0);
			}

			ev.HasMorePages = false;

			//draw a fullsize bitmap of the grid...
			int gridHeight = _grid.Model.RowHeights.GetTotal(0, _grid.Model.RowCount);
			int gridWidth = _grid.Model.ColWidths.GetTotal(0, _grid.Model.ColCount);



			Rectangle destRect = ev.MarginBounds;

			_rowIndex++;

			this._grid.TopRowIndex = _rowIndex;

			this._grid.TopRowIndex = _rowIndex;
						
			_grid.InternalSetTopRow(_rowIndex);

			float srcRatio =  (float)gridWidth / (float)gridHeight;
			float tarRatio = (float) destRect.Width / destRect.Height;
			int initialCount = 0;
			int rowIncrement = 0;
			if( tarRatio > srcRatio )
			{
				gridHeight = (int)(gridWidth/tarRatio);
				initialCount = _grid.RowIndexToVScrollPixelPos(_rowIndex);
				while (_grid.GetRowRangeHeight(_rowIndex, _rowIndex + rowIncrement) + _headerHeight < gridHeight
					&& (_rowIndex + rowIncrement) <= _grid.Model.RowCount)
				{
					rowIncrement += 1;
				}
				rowIncrement -= 1;

				gridHeight = _grid.GetRowRangeHeight(_rowIndex, _rowIndex + rowIncrement) + _headerHeight;

				_rowIndex += rowIncrement;
				if (_rowIndex < _grid.Model.RowCount)
				{
					ev.HasMorePages = true;
				}
			}

			Bitmap gridBM = new Bitmap(gridWidth, gridHeight);

			Graphics g = Graphics.FromImage(gridBM);

			this._grid.GridBounds = new Rectangle(0, 0, gridWidth, gridHeight);
			this._grid.LeftColIndex = 1;
			this._grid.DrawGrid(g);
			this._grid.ResetGridBounds();
	
			g.Dispose();


			//use the printer graphics
			g = ev.Graphics;

			//draw grid bitmap into the rect on the print page
			System.Drawing.GraphicsUnit gu = System.Drawing.GraphicsUnit.Point;
			RectangleF srcRect = gridBM.GetBounds(ref gu);


			//adjust destRect to make sizing proportional
			// you can comment this out this block if you don't want proportional fit
			srcRatio =  srcRect.Width / srcRect.Height;
			tarRatio = (float) destRect.Width / destRect.Height;

			if( tarRatio < srcRatio )
			{
				destRect.Height = (int) (destRect.Width / srcRatio);	
			}
			else
			{
				destRect.Width = (int) (destRect.Height * srcRatio);
			}
			//////////////endof proportional fit//////////////////////////

			g.DrawImage(gridBM,  destRect, srcRect, gu);

			//////////////Draw Header////////////////////////////////////
			string headerString = this._reportName + " " + this._reportStartDate + "-" + 
								  this._reportEndDate + "             " + System.DateTime.Now.ToLongDateString() +
								  " " + System.DateTime.Now.ToLongTimeString();
			g.DrawString(headerString, new Font("Arial", 10, FontStyle.Bold), new SolidBrush(Color.Black), new PointF(ev.MarginBounds.Left, ev.MarginBounds.Top/2));
			
			//////////////Draw Header////////////////////////////////////
			StringFormat format = new StringFormat();
			format.LineAlignment = StringAlignment.Near;
			format.Alignment = StringAlignment.Far;
			string pageString = "Page " + _pageCount;
			g.DrawString(pageString, new Font("Arial", 10, FontStyle.Bold), new SolidBrush(Color.Black), new PointF(ev.MarginBounds.Right, ev.MarginBounds.Top/2), format);

			if (ev.HasMorePages == false)
			{
				_grid.TopRowIndex = 1;
				_rowIndex = 0;
				_headerHeight = 0;
				_pageCount = 0;
			}
		}
	}
}
